package models

import (
	"eta_gn/eta_task/global"
	"fmt"
	"strings"
	"time"
)

// PptV2 表
type PptV2 struct {
	PptId            int       `gorm:"column:ppt_id;primaryKey;autoIncrement:true" description:"ppt的Id"`
	TemplateType     int       `gorm:"column:template_type" description:"模版类型"`
	BackgroundImg    string    `gorm:"column:background_img" description:"背景图片"`
	Title            string    `gorm:"column:title" description:"标题"`
	ReportType       string    `gorm:"column:report_type" description:"报告类型"`
	PptDate          string    `gorm:"column:ppt_date" description:"选择日期"`
	Content          string    `gorm:"column:content" description:"ppt内容"`
	PptUrl           string    `gorm:"column:ppt_url" description:"ppt下载地址"`
	PptxUrl          string    `gorm:"column:pptx_url" description:"pptx下载地址"`
	CreateTime       time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime       time.Time `gorm:"column:modify_time" description:"修改时间"`
	AdminId          int       `gorm:"column:admin_id" description:"系统用户id"`
	AdminRealName    string    `gorm:"column:admin_real_name" description:"系统用户名称"`
	PptVersion       int8      `gorm:"column:ppt_version" description:"是否ppt的旧版本；1：旧的，2：新的"`
	ReportId         int       `gorm:"column:report_id" description:"关联的报告ID"`
	ReportCode       string    `gorm:"column:report_code" description:"关联的报告code"`
	IsShare          int8      `gorm:"column:is_share" description:"是否分享，0：不分享，1：分享"`
	PublishTime      time.Time `gorm:"column:publish_time" description:"发布时间"`
	CoverContent     string    `gorm:"column:cover_content" description:"PPT内容-JSON"`
	PptPage          int       `gorm:"column:ppt_page" description:"PPT页数"`
	TitleSetting     string    `gorm:"column:title_setting" description:"PPT标题设置"`
	ClassifyId       int       `gorm:"column:classify_id" description:"报告分类ID"`
	AddType          int       `gorm:"column:add_type" description:"新增方式：1-新增报告；2-继承报告"`
	InheritReportId  int       `gorm:"column:inherit_report_id" description:"继承的报告ID"`
	CollaborateType  int       `gorm:"column:collaborate_type" description:"协作方式：1-个人；2-多人协作"`
	CollaborateUsers string    `gorm:"column:collaborate_users" description:"协作人IDs, 英文逗号分隔"`
	IsPublicPublish  int       `gorm:"column:is_public_publish" description:"是否公开发布：1-是；2-否"`
	SubmitTime       time.Time `gorm:"column:submit_time" description:"提交时间"`
	ApproveTime      time.Time `gorm:"column:approve_time" description:"审批时间"`
	ReportSource     int       `gorm:"column:report_source" description:"报告来源：1-系统内；2-智力共享"`
	OutReportId      string    `gorm:"column:out_report_id" description:"外部报告ID(或编码)"`
	State            int       `gorm:"column:state" description:"报告状态：1-未发布；2-已发布；3-待提交；4-待审批；5-已驳回；6-已通过"`
	TopicEndTime     time.Time `gorm:"column:topic_end_time" description:"课题结束时间"`
}

// PptReportQueryFields 除富文本的常用查询字段
var PptReportQueryFields = []string{
	"ppt_id", "title", "classify_id", "ppt_version", "pptx_url", "ppt_page", "title_setting", "state", "report_source", "publish_time", "submit_time", "approve_time", "create_time", "modify_time", "admin_id", "admin_real_name", "collaborate_type", "collaborate_users", "topic_end_time",
}

func (m *PptV2) TableName() string {
	return "ppt_v2"
}

func (m *PptV2) GetItemsByCondition(condition string, pars []interface{}, fieldArr []string, orderRule string) (items []*PptV2, err error) {
	fields := strings.Join(fieldArr, ",")
	if len(fieldArr) == 0 {
		fields = `*`
	}
	order := `ORDER BY create_time DESC`
	if orderRule != "" {
		order = ` ORDER BY ` + orderRule
	}
	sql := fmt.Sprintf(`SELECT %s FROM %s WHERE 1=1 %s %s`, fields, m.TableName(), condition, order)
	err = global.DmSQL["rddp"].Raw(sql, pars...).Find(&items).Error
	return
}
